/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2002,2003,2004 Gianni Tedesco <gianni@scaramanga.co.uk>
 * This program is released under the terms of the GNU GPL version 2
*/
#include "tcpip.h"

static void flow_match(struct detect *d, struct packet *p, int l)
{
	struct template_detect *x = (struct template_detect *)d;
	struct template_node *n;
	u_int32_t val;

	val = p->layer[l+1].flags;

	for(n=x->nodes; n; n=n->next) {
		if ( n->negate ^ ((val & n->u.val.max) == n->u.val.min) ) {
			if ( n->alert )
				alert_tag(p, n->alert, l);
			if ( n->result )
				n->result->match(n->result, p, l);
		}
	}

	if ( x->d.any )
		x->d.any->match(x->d.any, p, l);
}

static struct detect *flow_new(struct matcher *m)
{
	struct template_detect *x;

	x = calloc(1, sizeof(*x));
	if ( x == NULL )
		return NULL;

	x->d.match = flow_match;

	return (struct detect *)x;
}

static int stateless_validate(struct matcher *match, char *args,
				void **priv, struct modifier *m)
{
	struct template_node *p;

	/* 0/0 is what we want, no need to do anything */
	p = calloc(1, sizeof(*p));
	if ( p == NULL )
		return 0;

	*priv = p;

	return 1;
}

static int flow_validate(struct matcher *match, char *args,
			void **priv, struct modifier *m)
{
	u_int32_t min=FLAG_TCP_STATE;
	u_int32_t max=FLAG_TCP_STATE;
	struct template_node *p;
	char *tok, *cur, end=0;

	if ( !args )
		goto defaults;

	for(tok=cur=args; !end; cur++) {
		if ( *cur == ',' || *cur == 0 ) {
			if ( *cur == 0 )
				end = 1;
			*cur = 0;
			if ( !strcmp(tok, "to_server") ||
				!strcmp(tok, "from_client")) {
				max |= FLAG_TCP_2SVR;
				min |= FLAG_TCP_2SVR;
			}else if ( !strcmp(tok, "to_client") ||
				!strcmp(tok, "from_server") ) {
				max |= FLAG_TCP_2SVR;
				min &= ~FLAG_TCP_2SVR;
			}else if ( !strcmp(tok, "stateless") ) {
				max &= ~FLAG_TCP_STATE;
				min &= ~FLAG_TCP_STATE;
			}else if ( !strcmp(tok, "established") ) {
				max |= FLAG_TCP_SURE;
				min |= FLAG_TCP_SURE;
			}else if ( !strcmp(tok, "no_stream") ) {
				max |= FLAG_TCP_STREAM;
				min &= ~ FLAG_TCP_STREAM;
			}else
				return 0;
			tok = cur + 1;
		}
	}

defaults:

	p = calloc(1, sizeof(*p));
	if ( p == NULL )
		return 0;

	p->u.val.min = min;
	p->u.val.max = max;

	*priv = p;

	return 1;
}

struct matcher tcp_flow_matcher = {
	.validate = flow_validate,
	.free_priv = free,
	.new_node = flow_new,
	.insert_node = template_insert,
	.optimize = template_optimize,
};

struct matcher tcp_stateless_matcher = {
	.validate = stateless_validate,
	.free_priv = free,
	.new_node = flow_new,
	.insert_node = template_insert,
	.optimize = template_optimize,
};
