## Firestorm root directory
%define froot /var/lib/%{name}

## Firestorm Build Options: 1 for yes, 0 for no
%define	enable_gnome	1

%if %{enable_gnome}
%define gnome --enable-gnome
%else
%define gnome --disable-gnome
%endif

Summary:	A Network Intrusion Detection System (NIDS)
Name: 		firestorm
Version: 	0.6.0
Release: 	1
License: 	GPL
Group: 		System Environment/Daemons
Source: 	%{name}-%{version}.tar.bz2
Buildroot: 	%{_tmppath}/%{name}-%{version}-root

Requires:	libpcap

%description
Firestorm is an extremely high performance network intrusion detection
system (NIDS). At the moment it is just a sensor but plans are to include
real support for analysis, reporting, remote console and on-the-fly
sensor configuration. It is fully pluggable and hence extremely flexible.

%if %{enable_gnome}
%package gnome
Requires: 	libgnome >= 2.2.0, gtk2 >= 2.2.0, %{name} >= %{version}
Requires:	gnome-mime-data >= 2.4.0, gnome-vfs2 >= 2.4.0
BuildRequires: libgnome-devel, gtk2-devel, desktop-file-utils
BuildRequires:	gnome-vfs2-devel
Summary:	Gnome Firestorm Console
Group: 		Applications/System

%description gnome
Gnome Firestorm Console will open Firestorm's elog packet capture files
and display the contents in a nice gui.
Future versions will support filters, searching and sorting and conversion
to other data stream formats.
%endif

%prep
%setup -q

%build
%configure --sysconfdir=%{_sysconfdir} %{gnome}
make

%install
rm -rf %{buildroot}
make DESTDIR=%{buildroot} install

install -d -m 750 %{buildroot}%{_sysconfdir}/init.d
install -m 750 rc.firestorm %{buildroot}%{_sysconfdir}/init.d/firestorm

install -m 640 firestorm.rpm.conf %{buildroot}%{_sysconfdir}/firestorm.conf

# Remove *.la
rm -f %{buildroot}%{_libdir}/firestorm/*/*.la
rm -f %{buildroot}%{_libdir}/*.la

%if %{enable_gnome}
# Desktop file
DESKTOPS="firestorm.desktop"
for D in $DESKTOPS; do
	desktop-file-install --vendor %{desktop_vendor} \
	--dir %{buildroot}%{_datadir}/applications\
	--add-only-show-in=GNOME		\
	--add-category X-Red-Hat-Extras		\
	--add-category Application		\
	--add-category Network			\
	%{buildroot}%{_datadir}/applications/$D
	mv %{buildroot}%{_datadir}/applications/%{desktop_vendor}-$D %{buildroot}%{_datadir}/applications/$D
done
%endif


# Add user and group before installing
%pre
# old versions used this
userdel firestorm 2>/dev/null || true
groupdel firestorm 2>/dev/null || true

userdel fstorm 2>/dev/null || true
groupdel fstorm 2>/dev/null || true
groupadd fstorm || true
useradd -g fstorm -M -s /bin/false -d %{froot} fstorm || true

%post
chkconfig --add firestorm
chkconfig --level 235 firestorm on

%preun
chkconfig --level 235 firestorm del 2>/dev/null || true

%clean
rm -rf %{buildroot}

%files
%defattr(-,root,root)

# Configuration
%config(noreplace) %{_sysconfdir}/firestorm.conf
%{_sysconfdir}/init.d/firestorm

# Binaries
%attr(755,root,root)
%{_bindir}/firecat
%{_bindir}/firestat
%{_bindir}/firestorm-nids
%{_bindir}/firestorm-sniffer
%{_bindir}/stormwall

# Documentation
%defattr(-,root,root)
%doc %{_mandir}/man*/*
%doc ChangeLog NEWS README INSTALL CREDITS HACKING AUTHORS
%doc contrib/firestorm.vim

# Plugins
%dir %{_libdir}/firestorm/
%dir %{_libdir}/firestorm/capture
%{_libdir}/firestorm/capture/*.so
%dir %{_libdir}/firestorm/protocols
%{_libdir}/firestorm/protocols/*.so
%dir %{_libdir}/firestorm/detection
%{_libdir}/firestorm/detection/*.so
%dir %{_libdir}/firestorm/output
%{_libdir}/firestorm/output/*.so
%{_libdir}/libfirestorm.so.*
%{_libdir}/libfirestorm.so

# Firestorm root dir
%defattr(-,fstorm,fstorm)
%dir %{froot}
%config(noreplace) %{froot}/firestorm.rules
%{froot}/log
%{froot}/snort-rules

%if %{enable_gnome}
%files gnome
%defattr(-,root,root)
%attr(755,root,root) %{_bindir}/gnome-firestorm-console
%{_sysconfdir}/security/console.apps/gnome-firestorm-console
%{_sysconfdir}/pam.d/gnome-firestorm-console
%{_datadir}/applications/firestorm.desktop
%{_datadir}/application-registry/firestorm.applications
%{_datadir}/mime-info/firestorm.keys
%{_datadir}/pixmaps/firestorm.png
%endif
