/*
 * This file is part of dotscara
 * Copyright (c) 2003 Gianni Tedesco
 * Released under the terms of the GNU GPL version 2
 */
#ifndef __FOBUF_HEADER_INCLUDED__
#define __FOBUF_HEADER_INCLUDED__

/* for memcpy */
#include <string.h>

/** fobuf - File output buffer object */
struct fobuf {
	/* [ Do not change these fields, they are ABI */
	/** Underlying file descriptor */
	int fd;
	/** Virtual method table */
	const struct fobuf_ops *ops;

	/** @internal Pointer to base of buffer */
	char *buf;
	/** @internal Pointer to current position in buffer */
	char *ptr;
	/** @internal Amount of data remaining in the buffer */
	size_t buf_len;
	/** @internal Total size of the buffer */
	size_t buf_sz;
};

/** fobuf_ops - File output buffer virtual method table */
struct fobuf_ops {
	/** Write a 32bit integer from the file */
	int(*write32)(struct fobuf *b, u_int32_t val);
	/** Write a 16bit integer to the file */
	int(*write16)(struct fobuf *b, u_int16_t val);
};

int __public fobuf_attach(struct fobuf *b, int fd, int endian, size_t bufsz)
	__nonull(1);
int __public fobuf_detach(struct fobuf *b) __nonull(1);
int __public fobuf_close(struct fobuf *b) __nonull(1);
int __public fobuf_newfd(struct fobuf *b, int fd) __nonull(1);
static inline int fobuf_fd(struct fobuf *b) __nonull(1);
int __public fobuf_write(struct fobuf *b, const void *buf, size_t len)
	__nonull(1,2);
int fobuf_flush(struct fobuf *b) __nonull(1);

static inline int fobuf_fd(struct fobuf *b)
{
	return b->fd;
}

#endif /* __FOBUF_HEADER_INCLUDED__ */
