/*
* This file is part of Firestorm NIDS
* Copyright (c) 2002,2003,2004 Gianni Tedesco
* This program is released under the terms of the GNU GPL version 2
*
* This is for converting elog files in to firestorm packet structures,
* just as if they were generated from within firestorm itself.
*/
#include <firestorm.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <args.h>
#include <alert.h>
#include <signature.h>
#include <decode.h>
#include <exporter.h>

#include <elog.h>
#include <elog_read.h>

/* Free up any session data */
static void elog_free_sessions(struct packet *pkt)
{
	int i;
	struct layer *l;

	for(i = 0; i<pkt->llen; i++) {
		l = &pkt->layer[i];
		if ( l->session && l->proto && l->proto->free ) {
			l->proto->free(l->session);
		}
	}
}

/* Parse alert decode data */
static int elog_decode(struct packet *pkt, char *buf, int buflen)
{
	struct elog_dhdr *d;
	char *end = buf + buflen;
	struct proto *p;

	while( buf + sizeof(*d) <= end) {
		char *pnam, *pser, *dend;

		d=(struct elog_dhdr *)buf;

		/* Check the validity */
		if ( ((size_t)d->tot_len<<2) < sizeof(*d) )
			return 0;
		if ( ((size_t)d->tot_len<<2) < (d->name_len<<2) + sizeof(*d) )
			return 0;
		if ( buf + (d->tot_len<<2) > end )
			return 0;

		/* Work out where stuff is */
		pnam=(buf + sizeof(*d));
		pser = pnam+(d->name_len<<2);
		dend = buf+(d->tot_len<<2);

		if ( d->name_len == 0 ) {
			p = NULL;
		}else{
			p = decode_proto(pnam);
		}

		pkt->layer[pkt->llen].h.raw =
			pkt->base + ntohs(d->pkt_ofs);
		pkt->layer[pkt->llen].proto = p;
		pkt->layer[pkt->llen].flags = ntohl(d->flags);

		if ( p && p->deserialize && dend > pser ) {
			pkt->layer[pkt->llen].session = p->deserialize(
				pkt, pkt->llen, pser, dend - pser);
		}else{
			pkt->layer[pkt->llen].session = NULL;
		}

		if ( pkt->llen++ >= PKT_LAYERS )
			return 1;

		buf += d->tot_len<<2;
	}

	return 1;
}

/* Handler for alert records */
#if 0
int elog_pkt_alert(struct elog_pkt *p, proc_exporter fn, void *priv)
{
	struct generator gen;
	struct packet pkt;
	struct alert a;
	int ret;

	/* Fill in generator and alert structures */
	gen.name = p->generator;
	gen.t = NULL;
	a.alert = p->alert;
	a.sid = ntohl(p->hdr->sid);
	a.rev = ntohl(p->hdr->rev);
	a.priority = p->hdr->h.prio;
	a.gen = &gen;

	/* Fill in packet structure */
	pkt.len = ntohl(p->hdr->pkt_len);
	pkt.caplen = ntohl(p->hdr->pkt_caplen);
	pkt.llen = 0;
	pkt.base = p->data;
	pkt.end = p->data + pkt.caplen;
	pkt.time = (ntohl(p->hdr->h.tv_sec) * 1000000000) +
			(ntohl(p->hdr->h.tv_usec) * 1000);
	pkt.flags = ntohl(p->hdr->pflags) & FPMASK_PERSISTANT;
	pkt.alert = &a;

	/* Fill in decode data */
	if ( !elog_decode(&pkt, p->decode_buf,
		ntohs(p->hdr->decode_len)<<2) ) {
		return 0;
	}

	/* Call the output plugin */
	ret = fn(&pkt, priv);

	/* Free up session data */
	elog_free_sessions(&pkt);

	return ret;
}
#endif
