/*
 * This file is part of Firestorm NIDS
 * Copyright (c) 2003,2004 Gianni Tedesco
 * This program is released under the terms of the GNU GPL version 2
*/
#include <gnome.h>
#include <gtkhtml/gtkhtml.h>

#include <firestorm.h>
#include <alert.h>
#include <signature.h>
#include <decode.h>

#include <g_pktview.h>

GtkWidget *pktview_new(void)
{
	return gtk_html_new();
}

static void pktview_printf(GtkHTML *h, GtkHTMLStream *s, const char *fmt, ...)
	__printf(3,4);
static void pktview_printf(GtkHTML *h, GtkHTMLStream *s, const char *fmt, ...)
{
	va_list va;
	char buf[1024];
	int ret;

	va_start(va, fmt);
	ret = vsnprintf(buf, sizeof(buf), fmt, va);
	if ( ret < 0 )
		ret = 0;
	if ( sizeof(buf) <= (size_t)ret )
		mesg(M_ERR, "pktview_printf: Couldn't print message: %s", fmt);
	gtk_html_write(h, s, buf, strlen(buf));
	va_end(va);
}

static void hex_dump(GtkHTML *h, GtkHTMLStream *s, char *ptr,
			unsigned int len, unsigned int llen)
{
	unsigned int i, j;
	unsigned int line;
	unsigned char *tmp=(unsigned char *)ptr;

	for(j=0; j<len; ) {
		if ( j+llen > len ) {
			line=len-j;
		}else{
			line=llen;
		}

		pktview_printf(h, s, "%05X : ", j);

		for(i=0; i<llen; i++) {
			if ( i<line ) {
				if ( !isprint(tmp[i]) || (tmp[i]&0x80) ) {
					pktview_printf(h, s, ".");
				}else if ( isalnum(tmp[i]) ) {
					pktview_printf(h, s, "%c", tmp[i]);
				}else{
					pktview_printf(h, s, "&#x%.2x;", tmp[i]);
				}
			}else{
				pktview_printf(h, s, " ");
			}
		}

		for(i=0; i<line; i++) {
			pktview_printf(h, s, " %02X", tmp[i]);
		}

		j+=line;
		tmp+=line;
		pktview_printf(h, s, "\n");
	}
}

int pktview_show(struct packet *pkt, void *priv)
{
	struct tm *rtime;
	char tbuf[256];
	GtkHTML *h = GTK_HTML(priv);
	GtkHTMLStream *s;
	int i;
	time_t lt = pkt->time / TIMESTAMP_HZ;

	rtime = localtime(&lt);
	strftime(tbuf, sizeof(tbuf)-1, "%a %d %b %Y %H:%M:%S", rtime);

	s = gtk_html_begin(h);
	if ( s == NULL )
		return 1;

	pktview_printf(h, s, "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">");
	pktview_printf(h, s, "<html><head>");
	pktview_printf(h, s, "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=us-ascii\">");
	pktview_printf(h, s, "</head><body>");
	pktview_printf(h, s, "<p><table width=\"100%%\" bgcolor=\"#0000ff\" "
			"cellpadding=\"1\" cellspacing=\"0\"><tr><td>"
			"<table width=\"100%%\" bgcolor=\"#d0d0ff\">");
	pktview_printf(h, s, "<tr><td align=\"right\"><b>Alert:</b></td>"
			"<td width=\"100%%\">%s</td></tr>", pkt->alert->alert);
	pktview_printf(h, s, "<tr><td align=\"right\"><b>Date:</b></td>"
			"<td width=\"100%%\">%s.%9lu</td></tr>", tbuf,
			(unsigned long)(pkt->time % TIMESTAMP_HZ));
	pktview_printf(h, s, "<tr><td align=\"right\"><b>Signature:</b></td>"
			"<td width=\"100%%\">[%s:%u.%u]</td></tr>",
			pkt->alert->gen->name, pkt->alert->sid, pkt->alert->rev);

	for(i=0; i < pkt->llen; i++) {
		if ( !pkt->layer[i].proto ) {
			pktview_printf(h, s, "<tr><td align=\"right\"><b>Data:</b></td>"
					"<td width=\"100%%\">Application layer data "
					"(%u bytes)</td></tr>", pkt->end - pkt->layer[i].h.raw);
			continue;
		}

		if ( pkt->layer[i].proto->print ) {
			pkt->layer[i].proto->print(pkt, i, tbuf, sizeof(tbuf));
		}else{
			snprintf(tbuf, sizeof(tbuf), "(no information)");
		}

		pktview_printf(h, s, "<tr><td align=\"right\"><b>%s:</b></td>"
				"<td width=\"100%%\">%s</td></tr>",
				pkt->layer[i].proto->name, tbuf);
	}

	pktview_printf(h, s, "</table></td></tr></table>");

	pktview_printf(h, s, "<p><pre>");
	hex_dump(h, s, pkt->base, pkt->caplen, 16);
	pktview_printf(h, s, "</pre>");
	pktview_printf(h, s, "</body></html>");

	gtk_html_end(h, s, 0);

	return 1;
}
