/*
* This file is part of Firestorm NIDS
* Copyright (c) 2003,2004 Gianni Tedesco
* Released under the terms of the GNU GPL version 2
*
* Code for querying ordered indexes.
*/

#include <firestorm.h>
#include <dotscara/fobuf.h>
#include <dotscara/mpool.h>
#include <idx.h>
#include <q_idx.h>
#include <q_ordered.h>

#if 1
#define dprintf(...)
#else
#include <stdio.h>
#define dprintf printf
#endif

/* All records with keys defined by the range min-max inclusive */
int q_ordered_range(struct q_idx *idx, struct q_res *result,
			idx_key_t min, idx_key_t max)
{
	struct idx_onode *n=idx->map;
	idx_val_t *v1, *v2;
	unsigned int first,last;
	unsigned int imin, imax;
	unsigned int i;
	unsigned int maxf, maxl;	
	int ops=0;

	if ( idx->type != IDX_ORDERED_UINT32 )
		return 0;

	result->field=idx;
	result->num_keys=0;
	result->num_vals=0;
	result->val = NULL;

	if ( idx->num_keys == 0 )
		return 1;

	dprintf("range: %s: from %u to %u\n", idx->name, min, max);

	/* Binary search for the min key */
	maxf=first=0;
	maxl=last=idx->num_keys;
again1:
	ops++;
	i = first + ((last-first)/2);
	dprintf("%u:%u [%u] = %u\n", first, last, i, n[i].key);

	/* Use results here to narrow search for max key */
	if ( n[i].key > max ) {
		maxl = i;
	}else if ( n[i].key < max ) {
		maxf = i;
	}

	if ( min < n[i].key ) {
		last = i;
		if ( last-first )
			goto again1;
	}else if ( min > n[i].key ) {
		first = i+1;
		if ( last-first )
			goto again1;
	}

	while ( n[i].key < min && i < idx->num_keys )
		i++;

	if ( n[i].key > max )
		goto success;

	imin = i;
	dprintf("min: idx=%u (%u)\n", imin, n[imin].key);

	/* Binary search for the max key */
	first=maxf;
	last=maxl;
again2:
	ops++;
	i = first + ((last-first)/2);
	dprintf("%u:%u [%u] = %u\n", first, last, i, n[i].key);
	if ( max < n[i].key ) {
		last = i;
		if ( last-first )
			goto again2;
	}else if ( max > n[i].key ) {
		first = i+1;
		if ( last-first )
			goto again2;
	}

	while ( n[i].key > max && i )
		i--;

	if ( n[i].key > max )
		goto success;

	imax = i;
	dprintf("max: idx=%u (%u)\n", imax, n[imax].key);

	v1=idx->map + n[imin].ofs;
	v2=idx->map + n[imax].ofs;
	v2+=n[imax].num;

	result->num_keys=(imax-imin)+1;
	result->num_vals=v2-v1;
	result->val=idx->map + n[imin].ofs;
	result->node=&n[imin];

success:
	dprintf("found %u keys (%u vals) in %i ops\n",
		result->num_keys, result->num_vals, ops);

	return 1;
}

static int cmp(const void *a, const void *b)
{
	const struct idx_onode *aa=a, *bb=b;
	if ( aa->key < bb->key ) return -1;
	if ( aa->key > bb->key ) return 1;
	return 0;
}

int q_ordered_query(struct q_idx *idx, struct q_res *result, idx_key_t key)
{
	struct idx_onode n;
	struct idx_onode *res;

	if ( idx->type != IDX_ORDERED_UINT32 )
		return 0;

	if ( idx->num_keys == 0 ) {
		result->num_keys = 0;
		result->num_vals = 0;
		return 1;
	}

	result->field=idx;

	n.key = key;

	res=bsearch(&key, idx->map, idx->num_keys, sizeof(struct idx_onode), cmp);
	if ( !res ) {
		result->num_keys=0;
		result->num_vals=0;
		return 1;
	}

	result->num_keys=1;
	result->num_vals=res->num;
	result->val=idx->map + res->ofs;
	result->node=res;

	return 1;
}
